@@ -23,7 +23,6 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <config.h>
 #include <glib.h>
 #include <gmodule.h>
 #include <libgda/gda-config.h>
@@ -126,13 +125,38 @@ gda_config_parse_config_file (gchar *buffer, gint len)
 	xmlNodePtr cur;
 	GList *list = NULL;
 	gda_config_section *section;
+	gint sp_len;
+	const gchar *section_path = GDA_CONFIG_SECTION_DATASOURCES;
+	xmlFreeFunc old_free;
+	xmlMallocFunc old_malloc;
+	xmlReallocFunc old_realloc;
+	xmlStrdupFunc old_strdup;
 
 	g_return_val_if_fail (buffer != NULL, NULL);
 	g_return_val_if_fail (len != 0, NULL);
 
+	sp_len = strlen (section_path);
+
+	xmlMemGet (&old_free,
+		   &old_malloc,
+		   &old_realloc,
+		   &old_strdup);
+
+	xmlMemSetup (g_free,
+		     (xmlMallocFunc) g_malloc,
+		     (xmlReallocFunc) g_realloc,
+		     g_strdup);
+
+	xmlDoValidityCheckingDefaultValue = FALSE;
+	xmlKeepBlanksDefault(0);
+
 	doc = xmlParseMemory (buffer, len);
 	if (doc == NULL){
 		g_warning ("File empty or not well-formed.");
+		xmlMemSetup (old_free,
+			     old_malloc,
+			     old_realloc,
+			     old_strdup);
 		return NULL;
 	}
 
@@ -140,36 +164,53 @@ gda_config_parse_config_file (gchar *buffer, gint len)
 	if (cur == NULL){
 		g_warning ("Cannot get root element!");
 		xmlFreeDoc (doc);
+		xmlMemSetup (old_free,
+			     old_malloc,
+			     old_realloc,
+			     old_strdup);
 		return NULL;
 	}
 
 	if (strcmp (cur->name, "libgda-config") != 0){
 		g_warning ("root node != 'libgda-config' -> '%s'", cur->name);
 		xmlFreeDoc (doc);
+		xmlMemSetup (old_free,
+			     old_malloc,
+			     old_realloc,
+			     old_strdup);
 		return NULL;
 	}
 
-	cur = cur->xmlChildrenNode;
-	while (cur != NULL) {
-		if (!strcmp(cur->name, "section")){
-			section = g_new (gda_config_section, 1);
-			section->path = xmlGetProp (cur, "path");
-			if (section->path != NULL){
-				section->entries = gda_config_read_entries (cur);
-				list = g_list_append (list, section);
-			} else {
-				g_warning ("section without 'path'!");
-				g_free (section);
-			}
-		} else {
-			g_warning ("'section' expected, got '%s'. Ignoring...",
-				   cur->name);
+	for (cur = cur->xmlChildrenNode; cur != NULL; cur = cur->next) {
+		if (strcmp(cur->name, "section")) {
+			g_warning ("'section' expected, got '%s'. Ignoring...", cur->name);
+			continue;
 		}
 
-		cur = cur->next;
+		section = g_new (gda_config_section, 1);
+		section->path = xmlGetProp (cur, "path");
+		if (section->path != NULL &&
+		    !strncmp (section->path, section_path, sp_len)){
+			section->entries = gda_config_read_entries (cur);
+			if (section->entries == NULL) {
+				g_free (section->path);
+				g_free (section);
+				continue;
+			}
+			
+			list = g_list_append (list, section);
+		} else {
+			g_warning ("Ignoring section '%s'.", section->path);
+			g_free (section->path);
+			g_free (section);
+		}
 	}
 
 	xmlFreeDoc (doc);
+	xmlMemSetup (old_free,
+		     old_malloc,
+		     old_realloc,
+		     old_strdup);
 	return list;
 }
 
@@ -727,6 +768,7 @@ gda_config_remove_section (const gchar *path)
  * @path: path to the configuration entry
  *
  * Remove the given entry from the configuration database
+ * If the section is empty, also remove the section.
  */
 void
 gda_config_remove_key (const gchar *path)
@@ -751,23 +793,33 @@ gda_config_remove_key (const gchar *path)
 	entry = NULL;
 	cfg_client = get_config_client ();
 	section = gda_config_search_section (cfg_client->user, section_path);
-	if (section){
-		for (le = section->entries; le; le = le->next){
-			entry = le->data;
-			if (!strcmp (entry->name, ptr_last_dash + 1))
-				break;
+	if (section == NULL) {
+		g_free (section_path);
+		return;
+	}
 
-			entry = NULL;
-		}
+	for (le = section->entries; le; le = le->next){
+		entry = le->data;
+		if (!strcmp (entry->name, ptr_last_dash + 1))
+			break;
+
+		entry = NULL;
 	}
 
 	g_free (section_path);
-	if (entry != NULL){
-		section->entries = g_list_remove (section->entries, entry);
-		free_entry (entry, NULL);
-		write_config_file ();
-		do_notify (path);
+
+	if (entry == NULL)
+		return;
+
+	section->entries = g_list_remove (section->entries, entry);
+	free_entry (entry, NULL);
+	if (section->entries == NULL) {
+		cfg_client->user = g_list_remove (cfg_client->user, section);
+		free_section (section, NULL);
 	}
+
+	write_config_file ();
+	do_notify (path);
 }
 
 /**
@@ -1017,6 +1069,7 @@ gda_config_get_provider_list (void)
 				     name, NULL);
 		handle = g_module_open (path, G_MODULE_BIND_LAZY);
 		if (!handle) {
+			g_warning (_("Error: %s"), g_module_error ());
 			g_free (path);
 			continue;
 		}
