@@ -24,7 +24,6 @@
 
 #include <string.h>
 #include <stdlib.h>
-#include <config.h>
 #include <time.h>
 #include <glib/gdate.h>
 #include <glib/gmessages.h>
@@ -33,6 +32,8 @@
 #include <libgda/gda-intl.h>
 #include <libgda/gda-value.h>
 #include <libgda/gda-util.h>
+#include <libxml/parser.h>
+#include <libxml/tree.h>
 
 /*
  * Private functions
@@ -44,6 +45,11 @@ clear_value (GdaValue *value)
 	g_return_if_fail (value != NULL);
 
 	switch (value->type) {
+	case GDA_VALUE_TYPE_GOBJECT :
+		if (value->value.v_gobj)
+			g_object_unref (value->value.v_gobj);
+		value->value.v_gobj = NULL;
+		break;
 	case GDA_VALUE_TYPE_LIST :
 		g_list_foreach (value->value.v_list, (GFunc) gda_value_free, NULL);
 		g_list_free (value->value.v_list);
@@ -174,7 +180,7 @@ set_from_string (GdaValue *value, const gchar *as_string)
 			gda_value_set_double (value, dvalue);
 			retval = TRUE;
 		}
-		break;
+		break;	
 	case GDA_VALUE_TYPE_NUMERIC :
 		//FIXME: what test whould i do for numeric?
 		numeric.number = g_strdup (as_string);
@@ -201,6 +207,9 @@ set_from_string (GdaValue *value, const gchar *as_string)
 		make_point (&point, as_string);
 		gda_value_set_geometric_point (value, &point);
 		break;
+	case GDA_VALUE_TYPE_GOBJECT :
+		/* FIXME */
+		break;
 	case GDA_VALUE_TYPE_NULL :
 		gda_value_set_null (value);
 		break;
@@ -218,6 +227,7 @@ set_from_string (GdaValue *value, const gchar *as_string)
 		value->value.v_type = gda_type_from_string (as_string);
 		break;
 	case GDA_VALUE_TYPE_LIST : //FIXME
+	case GDA_VALUE_TYPE_MONEY : //FIXME
 	default :
 		gda_value_set_string (value, as_string);
 		retval = TRUE;
@@ -363,6 +373,25 @@ gda_value_new_geometric_point (const GdaGeometricPoint *val)
 	return value;
 }
 
+/**
+ * gda_value_new_gobject
+ * @val: value to set for the new #GdaValue.
+ *
+ * Make a new #GdaValue of type #GDA_VALUE_TYPE_GOBJECT with value @val.
+ *
+ * Returns: the newly created #GdaValue.
+ */
+GdaValue *
+gda_value_new_gobject (const GObject *val)
+{
+	GdaValue *value;
+
+	value = g_new0 (GdaValue, 1);
+	gda_value_set_gobject (value, val);
+
+	return value;
+}
+
 /**
  * gda_value_new_integer
  * @val: value to set for the new #GdaValue.
@@ -401,6 +430,25 @@ gda_value_new_list (const GdaValueList *val)
 	return value;
 }
 
+/**
+ + gda_value_new_money
+ * @val: value to set for the new #GdaValue.
+ *
+ * Make a new #GdaValue of type #GDA_VALUE_TYPE_MONEY with value @val.
+ *
+ * Returns: The newly created #GdaValue.
+ */
+GdaValue *
+gda_value_new_money (const GdaMoney *val)
+{
+	GdaValue *value;
+
+	value = g_new0 (GdaValue, 1);
+	gda_value_set_money (value, val);
+
+	return value;
+}
+
 /**
  * gda_value_new_numeric
  * @val: value to set for the new #GdaValue.
@@ -515,6 +563,39 @@ gda_value_new_timestamp (const GdaTimestamp *val)
 	return value;
 }
 
+/**
+ * gda_value_new_timestamp_from_timet
+ * @val: value to set for the new #GdaValue.
+ *
+ * Make a new #GdaValue of type #GDA_VALUE_TYPE_TIMESTAMP with value @val (of type
+ * time_t).
+ *
+ * Returns: the newly created #GdaValue.
+ */
+GdaValue *
+gda_value_new_timestamp_from_timet (time_t val)
+{
+	GdaValue *value;
+	struct tm *ltm;
+
+	value = g_new0 (GdaValue, 1);
+	ltm = localtime ((const time_t *) &val);
+	if (ltm) {
+		GdaTimestamp tstamp;
+		tstamp.year = ltm->tm_year;
+		tstamp.month = ltm->tm_mon;
+		tstamp.day = ltm->tm_mday;
+		tstamp.hour = ltm->tm_hour;
+		tstamp.minute = ltm->tm_min;
+		tstamp.second = ltm->tm_sec;
+		tstamp.fraction = 0;
+		tstamp.timezone = 0; /* FIXME */
+		gda_value_set_timestamp (value, (const GdaTimestamp *) &tstamp);
+	}
+
+	return value;
+}
+
 /**
  * gda_value_new_tinyint
  * @val: value to set for the new #GdaValue.
@@ -579,6 +660,38 @@ gda_value_new_from_string (const gchar *as_string, GdaValueType type)
 	return value;
 }
 
+/**
+ * gda_value_new_from_xml
+ * @xml: a XML node representing the value.
+ *
+ * Create a GdaValue from a XML representation of it. That XML
+ * node corresponds to the following string representation:
+ *    <value type="gdatype">value</value>
+ *
+ * Returns:  The newly created #GdaValue.
+ */
+GdaValue *
+gda_value_new_from_xml (const xmlNodePtr node)
+{
+	GdaValue *value;
+
+	g_return_val_if_fail (node != NULL, NULL);
+
+	/* parse the XML */
+	if (!node || !(node->name) || (node && strcmp (node->name, "value"))) 
+		return NULL;
+
+	value = g_new0 (GdaValue, 1);
+	if (!gda_value_set_from_string (value,
+					xmlNodeGetContent (node),
+					gda_type_from_string (xmlGetProp (node, "type")))) {
+		g_free (value);
+		value = NULL;
+	}
+
+	return value;
+}
+
 /**
  * gda_value_free
  * @value: the resource to free.
@@ -643,6 +756,7 @@ gda_value_is_number (GdaValue *value)
 	case GDA_VALUE_TYPE_BIGINT :
 	case GDA_VALUE_TYPE_DOUBLE :
 	case GDA_VALUE_TYPE_INTEGER :
+	case GDA_VALUE_TYPE_MONEY :
 	case GDA_VALUE_TYPE_NUMERIC :
 	case GDA_VALUE_TYPE_SINGLE :
 	case GDA_VALUE_TYPE_SMALLINT :
@@ -694,6 +808,10 @@ gda_value_copy (const GdaValue *value)
 	case GDA_VALUE_TYPE_GEOMETRIC_POINT :
 		memcpy (&copy->value.v_point, &value->value.v_point, sizeof (GdaGeometricPoint));
 		break;
+	case GDA_VALUE_TYPE_GOBJECT :
+		copy->value.v_gobj = value->value.v_gobj;
+		g_object_ref (copy->value.v_gobj);
+		break;
 	case GDA_VALUE_TYPE_INTEGER :
 		copy->value.v_integer = value->value.v_integer;
 		break;
@@ -705,6 +823,10 @@ gda_value_copy (const GdaValue *value)
 							    gda_value_copy (v));
 		}
 		break;
+	case GDA_VALUE_TYPE_MONEY :
+		copy->value.v_money.currency = g_strdup (value->value.v_money.currency);
+		copy->value.v_money.amount = value->value.v_money.amount;
+		break;
 	case GDA_VALUE_TYPE_NUMERIC :
 		memcpy (&copy->value.v_numeric, &value->value.v_numeric, sizeof (GdaNumeric));
 		break;
@@ -952,6 +1074,39 @@ gda_value_set_geometric_point (GdaValue *value, const GdaGeometricPoint *val)
 	value->value.v_point.y = val->y;
 }
 
+/**
+ * gda_value_get_gobject
+ * @value: a #GdaValue whose value we want to get.
+ *
+ * Gets the value stored in @value.
+ */
+const GObject *
+gda_value_get_gobject (GdaValue *value)
+{
+	g_return_val_if_fail (value != NULL, NULL);
+	g_return_val_if_fail (gda_value_isa (value, GDA_VALUE_TYPE_GOBJECT), NULL);
+	return (const GObject *) value->value.v_gobj;
+}
+
+/**
+ * gda_value_set_gobject
+ * @value: a #GdaValue that will store @val.
+ * @val: value to be stored in @value.
+ *
+ * Stores @val into @value.
+ */
+void
+gda_value_set_gobject (GdaValue *value, const GObject *val)
+{
+	g_return_if_fail (value != NULL);
+
+	clear_value (value);
+	value->type = GDA_VALUE_TYPE_GOBJECT;
+	value->value.v_gobj = (GObject *) val;
+	if (G_IS_OBJECT (val))
+		g_object_ref (value->value.v_gobj);
+}
+
 /**
  * gda_value_get_integer
  * @value: a #GdaValue whose value we want to get.
@@ -1017,6 +1172,41 @@ gda_value_set_list (GdaValue *value, const GdaValueList *val)
 	/* FIXME: implement */
 }
 
+/**
+ * gda_value_get_money
+ * @value: a #GdaValue whose value we want to get.
+ *
+ * Gets the value stored in @value.
+ * 
+ * Returns: the value contained in @value.
+ */
+const GdaMoney *
+gda_value_get_money (GdaValue *value)
+{
+	g_return_val_if_fail (value != NULL, NULL);
+	g_return_val_if_fail (gda_value_isa (value, GDA_VALUE_TYPE_MONEY), NULL);
+	return (const GdaMoney *) &value->value.v_money;
+}
+
+/**
+ * gda_value_set_money
+ * @value: a #GdaValue that will store @val.
+ * @val: value to be stored in @value.
+ *
+ * Stores @val into @value.
+ */
+void
+gda_value_set_money (GdaValue *value, const GdaMoney *val)
+{
+	g_return_if_fail (value != NULL);
+	g_return_if_fail (val != NULL);
+
+	clear_value (value);
+	value->type = GDA_VALUE_TYPE_MONEY;
+	value->value.v_money.currency = g_strdup (val->currency);
+	value->value.v_money.amount = val->amount;
+}
+
 /**
  * gda_value_set_null
  * @value: a #GdaValue that will store a value of type #GDA_VALUE_TYPE_NULL.
@@ -1376,12 +1566,18 @@ gda_value_set_from_value (GdaValue *value, const GdaValue *from)
 	case GDA_VALUE_TYPE_GEOMETRIC_POINT :
 		gda_value_set_geometric_point (value, gda_value_get_geometric_point ((GdaValue *) from));
 		break;
+	case GDA_VALUE_TYPE_GOBJECT :
+		gda_value_set_gobject (value, gda_value_get_gobject ((GdaValue *) from));
+		break;
 	case GDA_VALUE_TYPE_INTEGER :
 		gda_value_set_integer (value, gda_value_get_integer ((GdaValue *) from));
 		break;
 	case GDA_VALUE_TYPE_LIST :
 		gda_value_set_list (value, gda_value_get_list ((GdaValue *) from));
 		break;
+	case GDA_VALUE_TYPE_MONEY :
+		gda_value_set_money (value, gda_value_get_money ((GdaValue *) from));
+		break;
 	case GDA_VALUE_TYPE_NUMERIC :
 		gda_value_set_numeric (value, gda_value_get_numeric ((GdaValue *) from));
 		break;
@@ -1468,10 +1664,10 @@ gda_value_stringify (GdaValue *value)
 		retval = g_strdup_printf ("%d", gda_value_get_tinyint (value));
 		break;
 	case GDA_VALUE_TYPE_SINGLE:
-		retval = g_strdup_printf ("%f", gda_value_get_single (value));
+		retval = g_strdup_printf ("%.2f", gda_value_get_single (value));
 		break;
 	case GDA_VALUE_TYPE_DOUBLE:
-		retval = g_strdup_printf ("%f", gda_value_get_double (value));
+		retval = g_strdup_printf ("%.2f", gda_value_get_double (value));
 		break;
 	case GDA_VALUE_TYPE_TIME:
 		gdatime = gda_value_get_time (value);
@@ -1511,6 +1707,13 @@ gda_value_stringify (GdaValue *value)
 					  DBL_DIG,
 					  point->y);
 		break;
+	case GDA_VALUE_TYPE_GOBJECT :
+		if (value->value.v_gobj) {
+			retval = g_strdup_printf (_("(GObject of type '%s'"),
+						  g_type_name (G_TYPE_FROM_INSTANCE (value->value.v_gobj)));
+		} else
+			retval = g_strdup_printf (_("NULL GObject"));
+		break;
 	case GDA_VALUE_TYPE_LIST:
 		list = gda_value_get_list (value);
 		for (l = (GList *) list; l != NULL; l = l->next) {
@@ -1538,6 +1741,10 @@ gda_value_stringify (GdaValue *value)
 		else
 			retval = g_strdup ("");
 		break;
+	case GDA_VALUE_TYPE_MONEY :
+		retval = g_strdup_printf ("%s%f", value->value.v_money.currency,
+					  value->value.v_money.amount);
+		break;
 	case GDA_VALUE_TYPE_NUMERIC:
 		numeric = gda_value_get_numeric (value);
 		retval = g_strdup (numeric->number);
@@ -1597,6 +1804,12 @@ gda_value_compare (const GdaValue *value1, const GdaValue *value2)
 		retval = memcmp (&value1->value.v_point, &value2->value.v_point,
 				 sizeof (GdaGeometricPoint));
 		break;
+	case GDA_VALUE_TYPE_GOBJECT :
+		if (value1->value.v_gobj == value2->value.v_gobj)
+			retval = 0;
+		else
+			retval = (gint) value1->value.v_gobj - (gint) value2->value.v_gobj;
+		break;
 	case GDA_VALUE_TYPE_INTEGER :
 		retval = value1->value.v_integer - value2->value.v_integer;
 		break;
@@ -1611,6 +1824,13 @@ gda_value_compare (const GdaValue *value1, const GdaValue *value2)
 		if (retval == 0 && (l1 == NULL || l2 == NULL) && l1 != l2)
 			retval = (l1 == NULL) ? -1 : 1;
 		break;
+	case GDA_VALUE_TYPE_MONEY :
+		if (!strcmp (value1->value.v_money.currency ? value1->value.v_money.currency : "",
+			     value2->value.v_money.currency ? value2->value.v_money.currency : "")) {
+			retval = (gint) value1->value.v_double - value2->value.v_double;
+		} else
+			retval = -1; /* FIXME: do currency conversions to compare? */
+		break;
 	case GDA_VALUE_TYPE_NUMERIC :
 		retval = memcmp (&value1->value.v_numeric,
 				 &value2->value.v_numeric,
@@ -1649,3 +1869,66 @@ gda_value_compare (const GdaValue *value1, const GdaValue *value2)
 	return retval;
 }
 
+/*
+ * to_string
+ * 
+ * The exact reverse process of set_from_string(), almost the same as gda_value_stingify ()
+ * because of some localization with gda_value_stingify ().
+ */
+static gchar *
+to_string (GdaValue *value)
+{
+	const GdaTime *gdatime;
+	const GdaDate *gdadate;
+	const GdaTimestamp *timestamp;
+	const GdaGeometricPoint *point;
+	const GdaValueList *list;
+	const GdaNumeric *numeric;
+	GList *l;
+	GString *str = NULL;
+	gchar *retval = NULL;
+
+	g_return_val_if_fail (value != NULL, NULL);
+
+	switch (value->type){
+	case GDA_VALUE_TYPE_BOOLEAN:
+		if (gda_value_get_boolean (value))
+			retval = g_strdup ("true");
+		else
+			retval = g_strdup ("false");
+		break;
+	default:
+		retval = gda_value_stringify (value);
+	}
+        	
+	return retval;
+}
+
+
+/**
+ * gda_value_to_xml
+ * @value: a #GdaValue.
+ *
+ * Serialize the given #GdaValue to a XML node string.
+ *
+ * Returns: the XML node. Once not
+ * needed anymore, you should free it.
+ */
+xmlNodePtr
+gda_value_to_xml (GdaValue *value)
+{
+	gchar *valstr;
+	xmlNodePtr retval;
+
+	g_return_val_if_fail (value != NULL, NULL);
+
+	valstr = to_string (value);
+
+	retval = xmlNewNode (NULL, "value");
+	xmlSetProp (retval, "type", gda_type_to_string (value->type));
+	xmlNodeSetContent (retval, valstr);
+
+	g_free (valstr);
+
+	return retval;
+}
