@@ -24,8 +24,10 @@
 #if !defined(__gda_value_h__)
 #  define __gda_value_h__
 
+#include <time.h>
 #include <glib/glist.h>
 #include <glib/gmacros.h>
+#include <glib-object.h>
 #include <libxml/tree.h>
 
 G_BEGIN_DECLS
@@ -40,8 +42,10 @@ typedef enum {
 	GDA_VALUE_TYPE_DATE,
 	GDA_VALUE_TYPE_DOUBLE,
 	GDA_VALUE_TYPE_GEOMETRIC_POINT,
+	GDA_VALUE_TYPE_GOBJECT,
 	GDA_VALUE_TYPE_INTEGER,
 	GDA_VALUE_TYPE_LIST,
+	GDA_VALUE_TYPE_MONEY,
 	GDA_VALUE_TYPE_NUMERIC,
 	GDA_VALUE_TYPE_SINGLE,
 	GDA_VALUE_TYPE_SMALLINT,
@@ -64,6 +68,11 @@ typedef struct {
 	gdouble y;
 } GdaGeometricPoint;
 
+typedef struct {
+	gchar *currency;
+	gdouble amount;
+} GdaMoney;
+
 typedef struct {
 	gchar *number;
 	glong precision;
@@ -98,8 +107,10 @@ typedef struct {
 		GdaDate v_date;
 		gdouble v_double;
 		GdaGeometricPoint v_point;
+		GObject *v_gobj;
 		gint v_integer;
 		GdaValueList *v_list;
+		GdaMoney v_money;
 		GdaNumeric v_numeric;
 		gfloat v_single;
 		gshort v_smallint;
@@ -119,14 +130,17 @@ GdaValue     *gda_value_new_boolean (gboolean val);
 GdaValue     *gda_value_new_date (const GdaDate *val);
 GdaValue     *gda_value_new_double (gdouble val);
 GdaValue     *gda_value_new_geometric_point (const GdaGeometricPoint *val);
+GdaValue     *gda_value_new_gobject (const GObject *val);
 GdaValue     *gda_value_new_integer (gint val);
 GdaValue     *gda_value_new_list (const GdaValueList *val);
+GdaValue     *gda_value_new_money (const GdaMoney *val);
 GdaValue     *gda_value_new_numeric (const GdaNumeric *val);
 GdaValue     *gda_value_new_single (gfloat val);
 GdaValue     *gda_value_new_smallint (gshort val);
 GdaValue     *gda_value_new_string (const gchar *val);
 GdaValue     *gda_value_new_time (const GdaTime *val);
 GdaValue     *gda_value_new_timestamp (const GdaTimestamp *val);
+GdaValue     *gda_value_new_timestamp_from_timet (time_t val);
 GdaValue     *gda_value_new_tinyint (gchar val);
 GdaValue     *gda_value_new_type (GdaValueType val);
 GdaValue     *gda_value_new_from_string (const gchar *as_string,
@@ -153,11 +167,15 @@ gdouble       gda_value_get_double (GdaValue *value);
 void          gda_value_set_double (GdaValue *value, gdouble val);
 const GdaGeometricPoint *gda_value_get_geometric_point (GdaValue *value);
 void          gda_value_set_geometric_point (GdaValue *value, const GdaGeometricPoint *val);
+const GObject *gda_value_get_gobject (GdaValue *value);
+void          gda_value_set_gobject (GdaValue *value, const GObject *val);
 gint          gda_value_get_integer (GdaValue *value);
 void          gda_value_set_integer (GdaValue *value, gint val);
 const GdaValueList *gda_value_get_list (GdaValue *value);
 void          gda_value_set_list (GdaValue *value, const GdaValueList *val);
 void          gda_value_set_null (GdaValue *value);
+const GdaMoney *gda_value_get_money (GdaValue *value);
+void          gda_value_set_money (GdaValue *value, const GdaMoney *val);
 const GdaNumeric *gda_value_get_numeric (GdaValue *value);
 void          gda_value_set_numeric (GdaValue *value, const GdaNumeric *val);
 gfloat        gda_value_get_single (GdaValue *value);
