@@ -23,7 +23,6 @@
 #  define __gda_freetds_provider_h__
 
 #if defined(HAVE_CONFIG_H)
-#  include <config.h>
 #endif
 
 #include <libgda/gda-server-provider.h>
@@ -57,7 +56,7 @@ struct _GdaFreeTDSConnectionData {
 	gint          rc;         // rc code of last operation
 	GPtrArray     *msg_arr;   // array containing msgs from server
 	GPtrArray     *err_arr;   // array containing error msgs from server
-	GString       *database;  // database we are connected to
+	gchar         *database;  // database we are connected to
 	
 	TDSLOGIN      *login;     // tds login struct
 #ifdef HAVE_FREETDS_VER0_6X
@@ -65,7 +64,11 @@ struct _GdaFreeTDSConnectionData {
 #endif
 	TDSSOCKET     *tds;       // connection handle
 	TDSCONFIGINFO *config;    // tds config struct
-	gchar         *server_version; // Server identifier string
+
+	// Data just got at connection beginning
+	gchar         *server_id; // Server identifier/version string
+	gboolean      is_sybase;  // true if cnc to ASE, false for mssql
+	gulong        srv_ver;    // Server version
 };
 
 GType              gda_freetds_provider_get_type (void);
