@@ -22,7 +22,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
 #include <libgda/gda-data-model-array.h>
 #include <libgda/gda-intl.h>
 #include <stdlib.h>
@@ -184,7 +183,8 @@ gda_mysql_provider_open_connection (GdaServerProvider *provider,
         const gchar *t_password = NULL;
         const gchar *t_port = NULL;
         const gchar *t_unix_socket = NULL;
-        const gchar *t_flags = NULL;
+        const gchar *t_use_ssl = NULL;
+	unsigned int mysqlflags = 0;
 	MYSQL *mysql;
 #if MYSQL_VERSION_ID < 32200
         gint err;
@@ -198,17 +198,20 @@ gda_mysql_provider_open_connection (GdaServerProvider *provider,
 	/* get all parameters received */
 	t_host = gda_quark_list_find (params, "HOST");
 	t_db = gda_quark_list_find (params, "DATABASE");
-	t_user = gda_quark_list_find (params, "USERNAME");
+	t_user = gda_quark_list_find (params, "USER");
 	t_password = gda_quark_list_find (params, "PASSWORD");
 	t_port = gda_quark_list_find (params, "PORT");
 	t_unix_socket = gda_quark_list_find (params, "UNIX_SOCKET");
-	t_flags = gda_quark_list_find (params, "FLAGS");
+	t_use_ssl = gda_quark_list_find (params, "USE_SSL");
 
 	if (username)
 		t_user = username;
 	if (password)
 		t_password = password;
 
+	if (t_use_ssl && atoi (t_use_ssl) == 1)
+		mysqlflags |= CLIENT_SSL;
+
 	/* we can't have both a host/pair AND a unix_socket */
 	if ((t_host || t_port) && t_unix_socket) {
 		gda_connection_add_error_string (
@@ -219,10 +222,10 @@ gda_mysql_provider_open_connection (GdaServerProvider *provider,
 
 	/* provide the default of localhost:3306 if neither is provided */
 	if (!t_unix_socket) {
-		if (!t_port)
-			t_port = "3306";
 		if (!t_host)
 			t_host = "localhost";
+		else if (!t_port)
+			t_port = "3306";
 	}
 	
 	mysql = g_new0 (MYSQL, 1);
@@ -233,7 +236,7 @@ gda_mysql_provider_open_connection (GdaServerProvider *provider,
 #endif
 				    t_port ? atoi (t_port) : 0,
 				    t_unix_socket,
-				    t_flags ? atoi (t_flags) : 0);
+				    mysqlflags);
 	if (!mysql) {
 		error = gda_mysql_make_error (mysql);
 		gda_connection_add_error (cnc, error);
@@ -300,6 +303,7 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql)
 {
 	MYSQL *mysql;
 	gchar **arr;
+	GdaConnectionOptions options;
 
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
@@ -307,6 +311,8 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql)
 		return NULL;
 	}
 
+	options = gda_connection_get_options (cnc);
+
 	/* parse SQL string, which can contain several commands, separated by ';' */
 	arr = g_strsplit (sql, ";", 0);
 	if (arr) {
@@ -317,6 +323,24 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql)
 			MYSQL_RES *mysql_res;
 			GdaMysqlRecordset *recset;
 
+			/* if the connection is in read-only mode, just allow SELECT,
+			   SHOW commands */
+			if (options & GDA_CONNECTION_OPTIONS_READ_ONLY) {
+				gchar *s;
+
+				/* FIXME: maybe there's a better way of doing this? */
+				s = g_strstrip (g_strdup (arr[n]));
+				if (g_ascii_strncasecmp (s, "select", strlen ("select")) &&
+				    g_ascii_strncasecmp (s, "show", strlen ("show"))) {
+					gda_connection_add_error_string (
+						cnc, "Command '%s' cannot be executed in read-only mode", arr[n]);
+					break;
+				}
+
+				g_free (s);
+			}
+
+			/* execute the command on the server */
 			rc = mysql_real_query (mysql, arr[n], strlen (arr[n]));
 			if (rc != 0) {
 				gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
@@ -511,6 +535,11 @@ gda_mysql_provider_begin_transaction (GdaServerProvider *provider,
 		return FALSE;
 	}
 
+	if (gda_connection_get_options (cnc) & GDA_CONNECTION_OPTIONS_READ_ONLY) {
+		gda_connection_add_error_string (cnc, _("Transactions are not supported in read-only mode"));
+		return FALSE;
+	}
+
 	/* set isolation level */
 	switch (gda_transaction_get_isolation_level (xaction)) {
 	case GDA_TRANSACTION_ISOLATION_READ_COMMITTED :
@@ -563,6 +592,11 @@ gda_mysql_provider_commit_transaction (GdaServerProvider *provider,
 		return FALSE;
 	}
 
+	if (gda_connection_get_options (cnc) & GDA_CONNECTION_OPTIONS_READ_ONLY) {
+		gda_connection_add_error_string (cnc, _("Transactions are not supported in read-only mode"));
+		return FALSE;
+	}
+
 	rc = mysql_real_query (mysql, "COMMIT", strlen ("COMMIT"));
 	if (rc != 0) {
 		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
@@ -591,6 +625,11 @@ gda_mysql_provider_rollback_transaction (GdaServerProvider *provider,
 		return FALSE;
 	}
 
+	if (gda_connection_get_options (cnc) & GDA_CONNECTION_OPTIONS_READ_ONLY) {
+		gda_connection_add_error_string (cnc, _("Transactions are not supported in read-only mode"));
+		return FALSE;
+	}
+
 	rc = mysql_real_query (mysql, "ROLLBACK", strlen ("ROLLBACK"));
 	if (rc != 0) {
 		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
@@ -634,15 +673,15 @@ add_aggregate_row (GdaDataModelArray *recset, const gchar *str, const gchar *com
 	/* 2nd the unique id */
 	list = g_list_append (list, gda_value_new_string (str));
 	/* 3rd the owner */
-	list = g_list_append (list, gda_value_new_null ());
+	list = g_list_append (list, gda_value_new_string (NULL));
 	/* 4th the comments */ 
 	list = g_list_append (list, gda_value_new_string (comments));
 	/* 5th the return type */
-	list = g_list_append (list, gda_value_new_type (GDA_VALUE_TYPE_UNKNOWN));
+	list = g_list_append (list, gda_value_new_string (gda_type_to_string (GDA_VALUE_TYPE_UNKNOWN)));
 	/* 6th the argument type */
-	list = g_list_append (list, gda_value_new_type (GDA_VALUE_TYPE_UNKNOWN));
+	list = g_list_append (list, gda_value_new_string (gda_type_to_string (GDA_VALUE_TYPE_UNKNOWN)));
 	/* 7th the SQL definition */
-	list = g_list_append (list, gda_value_new_null ());
+	list = g_list_append (list, gda_value_new_string (NULL));
 
 	gda_data_model_append_row (GDA_DATA_MODEL (recset), list);
 
@@ -658,11 +697,14 @@ get_mysql_aggregates (GdaConnection *cnc, GdaParameterList *params)
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
 	/* create the recordset */
-	recset = (GdaDataModelArray *) gda_data_model_array_new (1);
-	//gda_server_recordset_model_set_field_defined_size (recset, 0, 32);
+	recset = (GdaDataModelArray *) gda_data_model_array_new (7);
 	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 0, _("Name"));
-	//gda_server_recordset_model_set_field_scale (recset, 0, 0);
-	//gda_server_recordset_model_set_field_gdatype (recset, 0, GDA_TYPE_STRING);
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 1, _("ID"));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 2, _("Owner"));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 3, _("Comments"));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 4, _("Return type"));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 5, _("Args types"));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 6, _("SQL"));
 
 	/* fill the recordset */
 	add_aggregate_row (recset, "abs", "comments");
