@@ -21,7 +21,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
 #include <libgda/gda-data-model-array.h>
 #include <libgda/gda-intl.h>
 #include <stdlib.h>
@@ -271,7 +270,7 @@ gda_oracle_provider_open_connection (GdaServerProvider *provider,
 	if (strlen(username) > 0)
 		ora_username = username;
 	else
-		ora_username = gda_quark_list_find (params, "USERNAME");
+		ora_username = gda_quark_list_find (params, "USER");
 
 	/* if the username isn't provided, try to find it in the DSN */
 	if (strlen(password) > 0)
@@ -814,13 +813,18 @@ gda_oracle_provider_begin_transaction (GdaServerProvider *provider,
 	g_return_val_if_fail (xaction != NULL, FALSE);
 
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
-	ora_xaction = g_new0 (GdaOracleTransaction, 1);
-
 	if (!priv_data) {
 		gda_connection_add_error_string (cnc, _("Invalid Oracle handle"));
 		return FALSE;
 	}
 
+	if (gda_connection_get_options (cnc) & GDA_CONNECTION_OPTIONS_READ_ONLY) {
+		gda_connection_add_error_string (cnc, _("Transactions are not supported in read-only mode"));
+		return FALSE;
+	}
+
+	ora_xaction = g_new0 (GdaOracleTransaction, 1);
+
 	/* allocate the Oracle transaction handle */
 	result = OCIHandleAlloc ((dvoid *) priv_data->henv,
 				(dvoid **) &(ora_xaction->txnhp),
@@ -891,11 +895,16 @@ gda_oracle_provider_commit_transaction (GdaServerProvider *provider,
 
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
 	ora_xaction = g_object_get_data (G_OBJECT (xaction), OBJECT_DATA_ORACLE_HANDLE);
-	if (!priv_data) {
+	if (!priv_data || !ora_xaction) {
 		gda_connection_add_error_string (cnc, _("Invalid Oracle handle"));
 		return FALSE;
 	}
 
+	if (gda_connection_get_options (cnc) & GDA_CONNECTION_OPTIONS_READ_ONLY) {
+		gda_connection_add_error_string (cnc, _("Transactions are not supported in read-only mode"));
+		return FALSE;
+	}
+
 	/* attach the transaction to the service context */
 	result = OCIAttrSet ((dvoid *) priv_data->hservice,
 				OCI_HTYPE_SVCCTX,
@@ -947,11 +956,16 @@ gda_oracle_provider_rollback_transaction (GdaServerProvider *provider,
 
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
 	ora_xaction = g_object_get_data (G_OBJECT (xaction), OBJECT_DATA_ORACLE_HANDLE);
-	if (!priv_data) {
+	if (!priv_data || !ora_xaction) {
 		gda_connection_add_error_string (cnc, _("Invalid Oracle handle"));
 		return FALSE;
 	}
 
+	if (gda_connection_get_options (cnc) & GDA_CONNECTION_OPTIONS_READ_ONLY) {
+		gda_connection_add_error_string (cnc, _("Transactions are not supported in read-only mode"));
+		return FALSE;
+	}
+
 	/* attach the transaction to the service context */
 	result = OCIAttrSet ((dvoid *) priv_data->hservice,
 				OCI_HTYPE_SVCCTX,
@@ -1515,6 +1529,37 @@ get_oracle_databases (GdaConnection *cnc, GdaParameterList *params)
 
 	return recset;
 }
+
+static GdaDataModel *
+get_oracle_tables (GdaConnection *cnc, GdaParameterList *params)
+{
+	GList *reclist;
+	GdaDataModel *recset;
+	gchar *sql = g_strdup_printf ("SELECT TABLE_NAME AS \"%s\","
+				      " OWNER AS \"%s\", "
+				      " NULL AS \"%s\", "
+				      " NULL AS \"SQL\" "
+				      " FROM ALL_TABLES "
+				      " ORDER BY TABLE_NAME",
+				      _("Name"), _("Owner"), _("Comments"));
+
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+
+	reclist = process_sql_commands (NULL, cnc, sql, NULL, 
+					GDA_COMMAND_OPTION_STOP_ON_ERRORS);
+
+	g_free (sql);
+
+	if (!reclist)
+		return NULL;
+
+	recset = GDA_DATA_MODEL (reclist->data);
+	g_list_free (reclist);
+
+	
+	return recset;
+}
+
 /*
  * get_oracle_objects
  *
@@ -1529,50 +1574,53 @@ get_oracle_databases (GdaConnection *cnc, GdaParameterList *params)
  * see any use for any user passed parameters at this point.
  */
 static GdaDataModel *
-get_oracle_objects (GdaConnection *cnc, GdaParameterList *params, GdaConnectionSchema schema)
+get_oracle_objects (GdaConnection *cnc, GdaParameterList *params, GdaConnectionSchema schema, gint nargs)
 {
 	GList *reclist;
 	GdaDataModel *recset;
+	gint cnt;
 	GdaParameterList *query_params = gda_parameter_list_new ();
-        gchar *sql = g_strdup ("SELECT OBJECT_NAME FROM USER_OBJECTS WHERE OBJECT_TYPE=:OBJ_TYPE");
+        GString *sql;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
+	sql = g_string_new ("SELECT OBJECT_NAME");
+	for (cnt = 1; cnt < nargs; cnt++)
+		sql = g_string_append (sql, ", NULL");
+
+	sql = g_string_append (sql, " FROM USER_OBJECTS WHERE OBJECT_TYPE=:OBJ_TYPE");
+
 	/* add the object type parameter */
 	switch (schema) {
 	case GDA_CONNECTION_SCHEMA_INDEXES:
 		gda_parameter_list_add_parameter (query_params, 
-			gda_parameter_new_string (":OBJ_TYPE", "INDEX"));
+						  gda_parameter_new_string (":OBJ_TYPE", "INDEX"));
 		break;
 	case GDA_CONNECTION_SCHEMA_PROCEDURES:
 		gda_parameter_list_add_parameter (query_params, 
-			gda_parameter_new_string (":OBJ_TYPE", "PROCEDURE"));
+						  gda_parameter_new_string (":OBJ_TYPE", "PROCEDURE"));
 		break;
 	case GDA_CONNECTION_SCHEMA_SEQUENCES:
 		gda_parameter_list_add_parameter (query_params, 
-			gda_parameter_new_string (":OBJ_TYPE", "SEQUENCE"));
-		break;
-	case GDA_CONNECTION_SCHEMA_TABLES:
-		gda_parameter_list_add_parameter (query_params, 
-			gda_parameter_new_string (":OBJ_TYPE", "TABLE"));
+						  gda_parameter_new_string (":OBJ_TYPE", "SEQUENCE"));
 		break;
 	case GDA_CONNECTION_SCHEMA_TRIGGERS:
 		gda_parameter_list_add_parameter (query_params, 
-			gda_parameter_new_string (":OBJ_TYPE", "TRIGGER"));
+						  gda_parameter_new_string (":OBJ_TYPE", "TRIGGER"));
 		break;
 	case GDA_CONNECTION_SCHEMA_VIEWS:
 		gda_parameter_list_add_parameter (query_params, 
-			gda_parameter_new_string (":OBJ_TYPE", "VIEW"));
+						  gda_parameter_new_string (":OBJ_TYPE", "VIEW"));
 		break;
 	default:
 		return NULL;
 	}
 
-	reclist = process_sql_commands (NULL, cnc, sql, query_params,
-			GDA_COMMAND_OPTION_STOP_ON_ERRORS);
+	reclist = process_sql_commands (NULL, cnc, sql->str, query_params,
+					GDA_COMMAND_OPTION_STOP_ON_ERRORS);
 
 	gda_parameter_list_free (query_params);
-	g_free (sql);
+	g_string_free (sql, TRUE);
 
 	if (!reclist)
 		return NULL;
@@ -1679,27 +1727,26 @@ gda_oracle_provider_get_schema (GdaServerProvider *provider,
 		recset = get_oracle_databases (cnc, params);
 		break;
 	case GDA_CONNECTION_SCHEMA_INDEXES :
-		recset = get_oracle_objects (cnc, params, schema);
+		recset = get_oracle_objects (cnc, params, schema, 1);
 		gda_data_model_set_column_title (recset, 0, _("Indexes"));
 		break;
 	case GDA_CONNECTION_SCHEMA_PROCEDURES :
-		recset = get_oracle_objects (cnc, params, schema);
+		recset = get_oracle_objects (cnc, params, schema, 8);
 		gda_data_model_set_column_title (recset, 0, _("Procedures"));
 		break;
 	case GDA_CONNECTION_SCHEMA_SEQUENCES :
-		recset = get_oracle_objects (cnc, params, schema);
+		recset = get_oracle_objects (cnc, params, schema, 4);
 		gda_data_model_set_column_title (recset, 0, _("Sequences"));
 		break;
 	case GDA_CONNECTION_SCHEMA_TABLES :
-		recset = get_oracle_objects (cnc, params, schema);
-		gda_data_model_set_column_title (recset, 0, _("Tables"));
+		recset = get_oracle_tables (cnc, params);
 		break;
 	case GDA_CONNECTION_SCHEMA_TRIGGERS :
-		recset = get_oracle_objects (cnc, params, schema);
+		recset = get_oracle_objects (cnc, params, schema, 1);
 		gda_data_model_set_column_title (recset, 0, _("Triggers"));
 		break;
 	case GDA_CONNECTION_SCHEMA_VIEWS :
-		recset = get_oracle_objects (cnc, params, schema);
+		recset = get_oracle_objects (cnc, params, schema, 4);
 		gda_data_model_set_column_title (recset, 0, _("Views"));
 		break;
 	default :
